﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using UPNPLib;
using System.IO;

namespace SatIp.UI.Player
{
    public partial class Form1 : Form
    {
        private DeviceCollector _deviceCollector;
        private RtspDevice _rtspDevice;
        private Boolean _isstreaming;
        private Timer _keepaLiveTimer;
        public Form1()
        {
            InitializeComponent();
            var source = GetStationsFromLocalFile_m3u(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData) + "\\SatIp.UI.Player\\PlayList.m3u");
            using (var enumerator = source.GetEnumerator())
            {
                while (enumerator.MoveNext())
                {
                    listBox1.DataSource = source;
                }
            }
        }
        private void KeepaLiveTimer_Tick(object sender, EventArgs e)
        {
            //try
            //{
                _rtspDevice.RtspSession.Options();
                //int level;
                //int quality;
                //_rtspDevice.RtspSession.Describe(out level, out quality);
                //tspgrLevel.Value = level;
                //tspgrQuality.Value = quality;
            //}
            //catch (Exception exception)
            //{
            //    Logger.Error(string.Format("{0}-{1}:{2}", "Form1", "_keepaLiveTimer_Tick", exception));
            //}

        }
        private void DeviceAdded(UPnPDevice device)
        {
            var newnode = treeView1.Nodes[0].Nodes.Add(device.UniqueDeviceName, device.FriendlyName);
            newnode.ToolTipText = device.Description;
            newnode.Tag = device;
            Console.WriteLine("found device with this {0} udn", device.UniqueDeviceName);
            if (treeView1.Nodes[0].IsExpanded != true)
                treeView1.Nodes[0].Expand();
        }

        private void SearchCompleted()
        {
            var tn = treeView1.Nodes[0].LastNode;
            treeView1.SelectedNode = tn;
            treeView1.Select();            
        }

        private void DeviceRemoved(string sUdn)
        {
            Console.WriteLine("should remove device with this {0} udn",sUdn);
            string[] sections = sUdn.Split(':');
            try
            {                
                if (_rtspDevice.UniqueDeviceName.Equals("uuid:" + sections[1]))
                {
                    StopPlayBack();
                    _keepaLiveTimer.Stop();
                    _rtspDevice.Dispose();
                    _isstreaming = false;
                }
            }
            finally { treeView1.Nodes[0].Nodes.RemoveByKey("uuid:" + sections[1]); }           
        }
        private void StartPlayBack(string mediafile)
        {
            axWindowsMediaPlayer1.URL = null;
            axWindowsMediaPlayer1.URL = mediafile;
            axWindowsMediaPlayer1.Ctlcontrols.play();
        }
        private void StopPlayBack()
        {
            axWindowsMediaPlayer1.Ctlcontrols.stop();
            axWindowsMediaPlayer1.URL = null;
            axWindowsMediaPlayer1.close();
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            _deviceCollector = new DeviceCollector();
            _deviceCollector.DeviceAdded += DeviceAdded;
            _deviceCollector.DeviceRemoved += DeviceRemoved;
            _deviceCollector.SearchCompleted += SearchCompleted;
        }        

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            Console.Write(e.Node.Tag);
            var finder = new UPnPDeviceFinderClass();
            var device = finder.FindByUDN(e.Node.Name);
            if (device != null)
            {
                _keepaLiveTimer = new Timer { Enabled = true };
                _keepaLiveTimer.Tick += KeepaLiveTimer_Tick;

                if ((_rtspDevice != null) && (!_rtspDevice.FiendlyName.Equals(device.FriendlyName)))
                {
                    _keepaLiveTimer.Stop();
                    _rtspDevice.Dispose();
                    _rtspDevice = new RtspDevice(device);
                    _keepaLiveTimer.Interval = _rtspDevice.RtspSession.RtspSessionTimeToLive;
                    _keepaLiveTimer.Start();
                    _isstreaming = false;
                }
                else
                {
                    _keepaLiveTimer.Stop();
                    _rtspDevice = new RtspDevice(device);
                    _keepaLiveTimer.Interval = _rtspDevice.RtspSession.RtspSessionTimeToLive;
                    _keepaLiveTimer.Start();
                }
                var service = (Service)listBox1.SelectedItem;
                if (!_isstreaming)
                {
                    _rtspDevice.RtspSession.Setup(service.ToString(), "unicast");
                    _rtspDevice.RtspSession.Play(String.Empty);
                    _isstreaming = true;
                    StartPlayBack(string.Format("rtp://{0}:{1}", _rtspDevice.RtspSession.Destination, _rtspDevice.RtspSession.ClientRtpPort));
                }
            }
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_rtspDevice != null)
            {
                var service = (Service)listBox1.SelectedItem;
                _rtspDevice.RtspSession.Play(service.ToString());
                StartPlayBack(string.Format("rtp://{0}:{1}", _rtspDevice.RtspSession.Destination, _rtspDevice.RtspSession.ClientRtpPort));
            }
        }
        private List<Service> GetStationsFromLocalFile_m3u(string fileName)
        {
            using (StreamReader reader = File.OpenText(fileName))
            {
                string[] strArray = reader.ReadToEnd().Split(new[] { "\n", "\r", "\n\r", "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
                var list = new List<Service>();
                if (strArray[0].Trim().ToUpper() == "#EXTM3U")
                {
                    var name = string.Empty;
                    var parameters = new string[15];
                    for (int i = 0; i < strArray.Length; i++)
                    {
                        if (strArray[i].StartsWith("#EXTINF", StringComparison.CurrentCultureIgnoreCase))
                        {
                            var strArray2 = strArray[i].Split(new[] { ":", "," }, StringSplitOptions.None);
                            if (strArray2.Length > 2)
                            {
                                name = strArray2[2];
                                parameters = strArray[++i].Split('&');
                            }
                            list.Add(new Service(name, parameters));
                        }
                        else if (strArray[i].StartsWith("# ", StringComparison.CurrentCultureIgnoreCase))
                        {
                            name = strArray[i].Substring(2);
                        }
                    }
                }
                return list;
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_rtspDevice != null)
                _rtspDevice.Dispose();
            if (_deviceCollector != null)
            {
                _deviceCollector.DeviceAdded -= DeviceAdded;
                _deviceCollector.DeviceRemoved -= DeviceRemoved;
                _deviceCollector.SearchCompleted -= SearchCompleted;
            }
        }      
    }
}
